/***************************************************************************
 *            callbacks.c
 *
 *  Wed Aug 16 18:13:26 2006
 *  Copyright  2006  Vincent Amouret
 *  vincent.amouret@gmail.com
 ****************************************************************************/

#include "callbacks.h"
#include "functions.h"
#include <glib/gprintf.h>
#include <glib/gi18n.h>
#include <string.h>

// Return codes of splashy_config
#define RETURN_OK 0
#define RETURN_ERROR 1
#define RETURN_NEED_ROOT 2

/**
 * First time the theme list is displayed.
 */
void
on_ThemeList_show (void)
{
  GtkWidget *TreeView = glade_xml_get_widget (xml, "ThemeList");

  initialise_parser ();

  /*
   * Fill the tree view
   */
  format_theme_list (TreeView);
  draw_theme_list (TreeView);
}

/**
 * Kill the main window.
 */
void
on_MainWindow_destroy (void)
{
  gtk_main_quit ();
}

/**
 * Select a theme.
 */
void
on_ThemeList_cursor_changed (gpointer user_data)
{
  GtkTreePath *path;
  GtkTreeViewColumn *column;
  GtkWidget *Statusbar =  glade_xml_get_widget (xml, "Statusbar");
  GtkWidget *MsgDialog = glade_xml_get_widget (xml, "Dialog");
  gdouble index = 0;
  gchar *options [4];
	
  gtk_tree_view_get_cursor (GTK_TREE_VIEW (user_data), &path, &column);
  index = g_strtod (gtk_tree_path_to_string (path), 0);
	
  options[0] = "splashy_config";
  options[1] = "--set-theme";
  options[2] = (gchar*)g_list_nth_data (ListOfThemes, index);
  options[3] = NULL;
  gint ret = splashy_config (options);

  switch (ret)
    {
    case RETURN_OK:
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "");
      // Display a dialog while we don't update-initramfs
      gtk_widget_show (MsgDialog);
      break;
    case RETURN_ERROR:  // Error
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("An error occured, the theme hasn't been selected"));
      break;
    case RETURN_NEED_ROOT:  // Not enough priviledges
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("You need super-user priviledges"));
      break;
    }
}

/**
 * Install a theme from a file chooser.
 */
void
on_InstallTheme_released (void)
{
  GtkWidget *TreeView = glade_xml_get_widget (xml, "ThemeList");
  GtkWidget *parent = glade_xml_get_widget (xml, "MainWindow");
  GtkWidget *Statusbar =  glade_xml_get_widget (xml, "Statusbar");
  GtkWidget *FileChooserDialog =  gtk_file_chooser_dialog_new (_("Select a theme"), GTK_WINDOW (parent), 
                                                               GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, 
                                                               GTK_RESPONSE_CANCEL, 
                                                               GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
	
  GtkFileFilter *filter = gtk_file_filter_new ();
  gint ret;
  gchar *options[4];
  /*
   * Set filter for .tar .tar.gz .tar.bz2 , the archives supported by splashy_config
   */
  gtk_file_filter_add_mime_type (filter, "application/x-tar");
  gtk_file_filter_add_mime_type (filter, "application/x-compressed-tar");
  gtk_file_filter_add_mime_type (filter, "application/x-bzip-compressed-tar");

  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserDialog), filter);
  ret = gtk_dialog_run (GTK_DIALOG (FileChooserDialog));


  if (ret == GTK_RESPONSE_OK)
    {
      gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserDialog));
		
      options[0] = "splashy_config";
      options[1] = "--install-theme";
      options[2] = g_strescape (filename, NULL);
      options[3] = NULL;
      ret = splashy_config (options);
      
      switch (ret)
        {
        case RETURN_OK:  // Don't tell anything, just redraw the list
          draw_theme_list (TreeView);
          gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "");
          break;
        case RETURN_ERROR:  // Error
          gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("An error occured, the theme hasn't been installed"));
          break;
        case RETURN_NEED_ROOT:  // Not enough priviledges
          gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("You need super-user priviledges to use this feature"));
          break;
        }
      
      g_free (filename);		
    }
	
  gtk_widget_destroy (FileChooserDialog);	
}

/**
 * Display the theme creation dialog then create it.
 */
void
on_CreateTheme_released (void)
{
  gboolean create = FALSE;
  GtkWidget *TreeView = glade_xml_get_widget (xml, "ThemeList");
  GtkWidget *CreateThemeDialog = glade_xml_get_widget (xml, "CreateThemeDialog");
  GtkWidget *Statusbar =  glade_xml_get_widget (xml, "Statusbar");
  /* Info */
  GtkWidget *EntryInfoName = glade_xml_get_widget (xml, "EntryInfoName");
  GtkWidget *EntryInfoVersion = glade_xml_get_widget (xml, "EntryInfoVersion");
  GtkWidget *EntryInfoDescription = glade_xml_get_widget (xml, "EntryInfoDescription");
  GtkWidget *EntryInfoUrls = glade_xml_get_widget (xml, "EntryInfoUrls");
  GtkWidget *EntryInfoAuthor = glade_xml_get_widget (xml, "EntryInfoAuthor");
  /* Background */
  GtkWidget *FileChooserButtonBoot = glade_xml_get_widget (xml, "FileChooserButtonBoot");
  GtkWidget *FileChooserButtonShutdown = glade_xml_get_widget (xml, "FileChooserButtonShutdown");
  GtkWidget *FileChooserButtonError = glade_xml_get_widget (xml, "FileChooserButtonError");
  GtkWidget *FileChooserButtonResume = glade_xml_get_widget (xml, "FileChooserButtonResume");
  GtkWidget *FileChooserButtonSuspend = glade_xml_get_widget (xml, "FileChooserButtonSuspend");
  GtkWidget *EntryBackgroundHeight = glade_xml_get_widget (xml, "EntryBackgroundHeight");
  GtkWidget *EntryBackgroundWidth = glade_xml_get_widget (xml, "EntryBackgroundWidth");
  GtkWidget *CheckbuttonBackgroundFadein = glade_xml_get_widget (xml, "CheckbuttonBackgroundFadein");
  GtkWidget *CheckbuttonBackgroundFadeout = glade_xml_get_widget (xml, "CheckbuttonBackgroundFadeout");
  GtkWidget *CheckbuttonVerbose = glade_xml_get_widget (xml, "CheckbuttonVerbose");
  /* Progress bar */
  GtkWidget *CheckbuttonProgressbarBoot = glade_xml_get_widget (xml, "CheckbuttonProgressbarBoot");
  GtkWidget *CheckbuttonProgressbarShutdown = glade_xml_get_widget (xml, "CheckbuttonProgressbarShutdown");
  GtkWidget *CheckbuttonProgressbarResume = glade_xml_get_widget (xml, "CheckbuttonProgressbarResume");
  GtkWidget *CheckbuttonProgressbarSuspend = glade_xml_get_widget (xml, "CheckbuttonProgressbarSuspend");
  GtkWidget *ComboboxProgressbarBoot = glade_xml_get_widget (xml, "ComboboxProgressbarBoot");
  GtkWidget *ComboboxProgressbarShutdown = glade_xml_get_widget (xml, "ComboboxProgressbarShutdown");
  GtkWidget *ComboboxProgressbarResume = glade_xml_get_widget (xml, "ComboboxProgressbarResume");
  GtkWidget *ComboboxProgressbarSuspend = glade_xml_get_widget (xml, "ComboboxProgressbarSuspend");
  GtkWidget *SpinbuttonProgressbarX = glade_xml_get_widget (xml, "SpinbuttonProgressbarX");
  GtkWidget *SpinbuttonProgressbarY = glade_xml_get_widget (xml, "SpinbuttonProgressbarY");
  GtkWidget *SpinbuttonProgressbarWidth = glade_xml_get_widget (xml, "SpinbuttonProgressbarWidth");
  GtkWidget *SpinbuttonProgressbarHeight = glade_xml_get_widget (xml, "SpinbuttonProgressbarHeight");
  GtkWidget *ColorbuttonProgressbarForeground = glade_xml_get_widget (xml, "ColorbuttonProgressbarForeground");
  GtkWidget *ColorbuttonProgressbarBackground = glade_xml_get_widget (xml, "ColorbuttonProgressbarBackground");
  GtkWidget *SpinbuttonProgressbarForegroundAlpha = glade_xml_get_widget (xml, "SpinbuttonProgressbarForegroundAlpha");
  GtkWidget *SpinbuttonProgressbarBackgroundAlpha = glade_xml_get_widget (xml, "SpinbuttonProgressbarBackgroundAlpha");
  GtkWidget *CheckbuttonProgressbarBorder = glade_xml_get_widget (xml, "CheckbuttonProgressbarBorder");
  GtkWidget *ColorbuttonProgressbarBorder = glade_xml_get_widget (xml, "ColorbuttonProgressbarBorder");
  GtkWidget *SpinbuttonProgressbarBorderAlpha = glade_xml_get_widget (xml, "SpinbuttonProgressbarBorderAlpha");
  /* Text box */
  GtkWidget *CheckbuttonTextbox = glade_xml_get_widget (xml, "CheckbuttonTextbox");
  GtkWidget *SpinbuttonTextboxX = glade_xml_get_widget (xml, "SpinbuttonTextboxX");
  GtkWidget *SpinbuttonTextboxY = glade_xml_get_widget (xml, "SpinbuttonTextboxY");
  GtkWidget *SpinbuttonTextboxWidth = glade_xml_get_widget (xml, "SpinbuttonTextboxWidth");
  GtkWidget *SpinbuttonTextboxHeight = glade_xml_get_widget (xml, "SpinbuttonTextboxHeight");
  GtkWidget *ColorbuttonTextbox = glade_xml_get_widget (xml, "ColorbuttonTextbox");
  GtkWidget *SpinbuttonTextboxAlpha = glade_xml_get_widget (xml, "SpinbuttonTextboxAlpha");
  GtkWidget *CheckbuttonTextboxBorder = glade_xml_get_widget (xml, "CheckbuttonTextboxBorder");
  GtkWidget *ColorbuttonTextboxBorder = glade_xml_get_widget (xml, "ColorbuttonTextboxBorder");
  GtkWidget *SpinbuttonTextboxBorderAlpha = glade_xml_get_widget (xml, "SpinbuttonTextboxBorderAlpha");
  GtkWidget *FileChooserButtonFont = glade_xml_get_widget (xml, "FileChooserButtonFont");
  GtkWidget *SpinbuttonFontSize = glade_xml_get_widget (xml, "SpinbuttonFontSize");
  GtkWidget *ColorbuttonTextboxText = glade_xml_get_widget (xml, "ColorbuttonTextboxText");
  GtkWidget *SpinbuttonTextboxTextAlpha = glade_xml_get_widget (xml, "SpinbuttonTextboxTextAlpha");

  /*
   * Set the filter on PNG files
   */
  GtkFileFilter *filter = gtk_file_filter_new ();
  gtk_file_filter_add_mime_type (filter, "image/png");
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonBoot), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonShutdown), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonError), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonResume), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonSuspend), filter);
					 
  /*
   * Set the filter on TTF files
   */
  GtkFileFilter *filter_font = gtk_file_filter_new ();
  gtk_file_filter_add_mime_type (filter_font, "application/x-font-ttf");
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (FileChooserButtonFont), filter_font);
					 
  /*
   * Set the combo boxes on "Forward" or "Backward"
   * depending the usual look
   */
  gtk_combo_box_set_active (GTK_COMBO_BOX (ComboboxProgressbarBoot), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (ComboboxProgressbarShutdown), 1);
  gtk_combo_box_set_active (GTK_COMBO_BOX (ComboboxProgressbarResume), 0);
  gtk_combo_box_set_active (GTK_COMBO_BOX (ComboboxProgressbarSuspend), 1);

  /*
   * FIXME
   * Find a way to clean the values on show or on hide instead of to set
   * the values to an empty string or 0.
   */
  /* Info */
  gtk_entry_set_text (GTK_ENTRY (EntryInfoName), "");
  gtk_entry_set_text (GTK_ENTRY (EntryInfoVersion), "");
  gtk_entry_set_text (GTK_ENTRY (EntryInfoDescription), "");
  gtk_entry_set_text (GTK_ENTRY (EntryInfoUrls), "");
  gtk_entry_set_text (GTK_ENTRY (EntryInfoAuthor), "");
  /* Background */
  gtk_entry_set_text (GTK_ENTRY (EntryBackgroundWidth), "");
  gtk_entry_set_text (GTK_ENTRY (EntryBackgroundHeight), "");
  /* Progress bar */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarX), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarY), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarWidth), 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarHeight), 1);
  /* Text box */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonTextboxX), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonTextboxY), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonTextboxWidth), 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (SpinbuttonTextboxHeight), 1);
  
  gint ret = gtk_dialog_run (GTK_DIALOG (CreateThemeDialog));
	
  if (ret == GTK_RESPONSE_APPLY)
    {
      create = TRUE;
      /* Info */
      gchar *name = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryInfoName)));
      gchar *version = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryInfoVersion)));
      gchar *description = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryInfoDescription)));
      gchar *urls = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryInfoUrls)));
      gchar *author = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryInfoAuthor)));
      /* Background */
      gchar *boot_image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonBoot));
      gchar *shutdown_image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonShutdown));
      gchar *error_image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonError));
      gchar *resume_image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonResume));
      gchar *suspend_image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonSuspend));
      gchar *width = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryBackgroundWidth)));
      gchar *height = g_strdup (gtk_entry_get_text (GTK_ENTRY (EntryBackgroundHeight)));
      gboolean b_fadein = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonBackgroundFadein));
      gchar *fadein = b_fadein ? "yes" : "no";
      gboolean b_fadeout = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonBackgroundFadeout));
      gchar *fadeout = b_fadeout ? "yes" : "no";
      gboolean b_verbose = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonVerbose));
      gchar *verbose = b_verbose ? "yes" : "no";
      /* Progress bar */
      gboolean b_pb_boot = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonProgressbarBoot));
      gchar *pb_boot = b_pb_boot ? "yes" : "no";
      gboolean b_pb_shutdown = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonProgressbarShutdown));
      gchar *pb_shutdown = b_pb_shutdown ? "yes" : "no";
      gboolean b_pb_resume = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonProgressbarResume));
      gchar *pb_resume = b_pb_resume ? "yes" : "no";
      gboolean b_pb_suspend = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonProgressbarSuspend));
      gchar *pb_suspend = b_pb_suspend ? "yes" : "no";
      gint i_pb_dir_boot = gtk_combo_box_get_active (GTK_COMBO_BOX (ComboboxProgressbarBoot));
      gchar *pb_dir_boot = (i_pb_dir_boot == 1) ? "backward" : "forward";
      gint i_pb_dir_shutdown = gtk_combo_box_get_active (GTK_COMBO_BOX (ComboboxProgressbarShutdown));
      gchar *pb_dir_shutdown = (i_pb_dir_shutdown == 1) ? "backward" : "forward";
      gint i_pb_dir_resume = gtk_combo_box_get_active (GTK_COMBO_BOX (ComboboxProgressbarResume));
      gchar *pb_dir_resume = (i_pb_dir_resume == 1) ? "backward" : "forward";
      gint i_pb_dir_suspend = gtk_combo_box_get_active (GTK_COMBO_BOX (ComboboxProgressbarSuspend));
      gchar *pb_dir_suspend = (i_pb_dir_suspend == 1) ? "backward" : "forward";
      guint pb_x = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarX));
      guint pb_y = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarY));
      guint pb_width = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarWidth));
      guint pb_height = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarHeight));
      GdkColor pb_foreground;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonProgressbarForeground), &pb_foreground);
      GdkColor pb_background;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonProgressbarBackground), &pb_background);
      guint pb_foreground_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarForegroundAlpha));
      guint pb_background_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarBackgroundAlpha));
      gboolean b_pb_border = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonProgressbarBorder));
      gchar *pb_border = b_pb_border ? "yes" : "no";
      GdkColor pb_border_color;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonProgressbarBorder), &pb_border_color);
      guint pb_border_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonProgressbarBorderAlpha));
      /* Text box */
      gboolean b_textbox = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonTextbox));
      gchar *textbox = b_textbox ? "yes" : "no";
      guint textbox_x = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxX));
      guint textbox_y = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxY));
      guint textbox_width = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxWidth));
      guint textbox_height = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxHeight));
      GdkColor textbox_color;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonTextbox), &textbox_color);
      guint textbox_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxAlpha));
      gboolean b_textbox_border = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (CheckbuttonTextboxBorder));
      gchar *textbox_border = b_textbox_border ? "yes" : "no";
      GdkColor textbox_border_color;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonTextboxBorder), &textbox_border_color);
      guint textbox_border_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxBorderAlpha));
      gchar *textbox_font = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (FileChooserButtonFont));
      guint textbox_font_size = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonFontSize));		           
      GdkColor textbox_text_color;
      gtk_color_button_get_color (GTK_COLOR_BUTTON (ColorbuttonTextboxText), &textbox_text_color);
      guint textbox_text_alpha = gtk_spin_button_get_value (GTK_SPIN_BUTTON (SpinbuttonTextboxTextAlpha));
		
      /* TODO
       * If the name and the pictures are not NULL then allow the apply 
       * button to be pressed.
       * Check the width and height >=0, then change the max value of the
       * spin buttons of progressbar X and Y (max_x=width, max_y=height).
       */
      if (name == NULL)
        {
          g_printerr ("ERROR: Name is NULL\n");
          create = FALSE;
        }
      if (boot_image == NULL)
        {
          g_printerr ("ERROR: Boot image is NULL\n");
          create = FALSE;
        }
      if (shutdown_image == NULL)
        {
          g_printerr ("ERROR: Shutdown image is NULL\n");
          create = FALSE;
        }
      if (resume_image == NULL)
        {
          g_printerr ("ERROR: Resume image is NULL\n");
          create = FALSE;
        }
      if (suspend_image == NULL)
        {
          g_printerr ("ERROR: Suspend image is NULL\n");
          create = FALSE;
        }
      gint i = 0;	
      gchar *options[123];
      if (create == TRUE)
        {			
          options[i++] = g_strdup ("splashy_config");
          options[i++] = g_strdup ("--create-theme");
			
          options[i++] = g_strdup ("--name");
          options[i++] = name;
          options[i++] = g_strdup ("--boot-image");
          options[i++] = boot_image;
          options[i++] = g_strdup ("--shutdown-image");
          options[i++] = shutdown_image;
          options[i++] = g_strdup ("--error-image");
          options[i++] = error_image;
          options[i++] = g_strdup ("--resume-image");
          options[i++] = resume_image;
          options[i++] = g_strdup ("--suspend-image");
          options[i++] = suspend_image;
			
          if (version)
            {
              options[i++] = g_strdup ("--version");
              options[i++] = version;
            }
          if (description)
            {
              options[i++] = g_strdup ("--description");
              options[i++] = description;
            }
          if (urls)
            {
              options[i++] = g_strdup ("--urls");
              options[i++] = urls;
            }
          if (author)
            {
              options[i++] = g_strdup ("--author");
              options[i++] = author;
            }
          if (width > 0 && height > 0)
            {
              options[i++] = g_strdup ("--background-width");
              options[i++] = g_strdup_printf ("%d", (gint)g_strtod (width, NULL));
              options[i++] = g_strdup ("--background-height");
              options[i++] = g_strdup_printf ("%d", (gint)g_strtod (height, NULL));
            }
          if (fadein)
            {
              options[i++] = g_strdup ("--fadein");
              options[i++] = g_strdup (fadein);
            }
          if (fadeout)
            {
              options[i++] = g_strdup ("--fadeout");
              options[i++] = g_strdup (fadeout);
            }
          if (pb_boot)
            {
              options[i++] = g_strdup ("--progressbar-visibility-boot");
              options[i++] = g_strdup (pb_boot);
            }
          if (pb_shutdown)
            {
              options[i++] = g_strdup ("--progressbar-visibility-shutdown");
              options[i++] = g_strdup (pb_shutdown);
            }
          if (pb_resume)
            {
              options[i++] = g_strdup ("--progressbar-visibility-resume");
              options[i++] = g_strdup (pb_resume);
            }
          if (pb_suspend)
            {
              options[i++] = g_strdup ("--progressbar-visibility-suspend");
              options[i++] = g_strdup (pb_suspend);
            }
          if (pb_dir_boot)
            {
              options[i++] = g_strdup ("--progressbar-direction-boot");
              options[i++] = g_strdup (pb_dir_boot);
            }
          if (pb_dir_shutdown)
            {
              options[i++] = g_strdup ("--progressbar-direction-shutdown");
              options[i++] = g_strdup (pb_dir_shutdown);
            }
          if (pb_dir_resume)
            {
              options[i++] = g_strdup ("--progressbar-direction-resume");
              options[i++] = g_strdup (pb_dir_resume);
            }
          if (pb_dir_suspend)
            {
              options[i++] = g_strdup ("--progressbar-direction-suspend");
              options[i++] = g_strdup (pb_dir_suspend);
            }
			
          options[i++] = g_strdup ("--progressbar-x");
          options[i++] = g_strdup_printf ("%d", pb_x);
          options[i++] = g_strdup ("--progressbar-y");
          options[i++] = g_strdup_printf ("%d", pb_y);
			
          if (pb_width > 0)
            {
              options[i++] = g_strdup ("--progressbar-width");
              options[i++] = g_strdup_printf ("%d", pb_width);
            }
          if (pb_height > 0)
            {
              options[i++] = g_strdup ("--progressbar-height");
              options[i++] = g_strdup_printf ("%d", pb_height);
            }
			
          options[i++] = g_strdup ("--progressbar-red");
          options[i++] = g_strdup_printf ("%d", pb_foreground.red / 257);
          options[i++] = g_strdup ("--progressbar-green");
          options[i++] = g_strdup_printf ("%d", pb_foreground.green / 257);
          options[i++] = g_strdup ("--progressbar-blue");
          options[i++] = g_strdup_printf ("%d", pb_foreground.blue / 257);
          options[i++] = g_strdup ("--progressbar-alpha");
          options[i++] = g_strdup_printf ("%d", pb_foreground_alpha);
          options[i++] = g_strdup ("--progressbar-background-red");
          options[i++] = g_strdup_printf ("%d", pb_background.red / 257);
          options[i++] = g_strdup ("--progressbar-background-green");
          options[i++] = g_strdup_printf ("%d", pb_background.green / 257);
          options[i++] = g_strdup ("--progressbar-background-blue");
          options[i++] = g_strdup_printf ("%d", pb_background.blue / 257);
          options[i++] = g_strdup ("--progressbar-background-alpha");
          options[i++] = g_strdup_printf ("%d", pb_background_alpha);
			
          if (pb_border)
            {
              options[i++] = g_strdup ("--progressbar-border-show");
              options[i++] = g_strdup (pb_border);
			
              options[i++] = g_strdup ("--progressbar-border-red");
              options[i++] = g_strdup_printf ("%d", pb_border_color.red / 257);
              options[i++] = g_strdup ("--progressbar-border-green");
              options[i++] = g_strdup_printf ("%d", pb_border_color.green / 257);
              options[i++] = g_strdup ("--progressbar-border-blue");
              options[i++] = g_strdup_printf ("%d", pb_border_color.blue / 257);
              options[i++] = g_strdup ("--progressbar-border-alpha");
              options[i++] = g_strdup_printf ("%d", pb_border_alpha);
            }
          if (textbox)
            {
              options[i++] = g_strdup ("--textbox-show");
              options[i++] = g_strdup (textbox);
				
              options[i++] = g_strdup ("--textbox-x");
              options[i++] = g_strdup_printf ("%d", textbox_x);
              options[i++] = g_strdup ("--textbox-y");
              options[i++] = g_strdup_printf ("%d", textbox_y);
				
              if (textbox_width > 0)
                {
                  options[i++] = g_strdup ("--textbox-width");
                  options[i++] = g_strdup_printf ("%d", textbox_width);
                }
              if (textbox_height > 0)
                {
                  options[i++] = g_strdup ("--textbox-height");
                  options[i++] = g_strdup_printf ("%d", textbox_height);
                }
				
              options[i++] = g_strdup ("--textbox-red");
              options[i++] = g_strdup_printf ("%d", textbox_color.red / 257);
              options[i++] = g_strdup ("--textbox-green");
              options[i++] = g_strdup_printf ("%d", textbox_color.green / 257);
              options[i++] = g_strdup ("--textbox-blue");
              options[i++] = g_strdup_printf ("%d", textbox_color.blue / 257);
              options[i++] = g_strdup ("--textbox-alpha");
              options[i++] = g_strdup_printf ("%d", textbox_alpha);
				
              if (textbox_font)
                {
                  options[i++] = g_strdup ("--text-font-file");
                  options[i++] = textbox_font;
                }
				
              options[i++] = g_strdup ("--text-font-height");
              options[i++] = g_strdup_printf ("%d", textbox_font_size);
              options[i++] = g_strdup ("--text-font-red");
              options[i++] = g_strdup_printf ("%d", textbox_text_color.red / 257);
              options[i++] = g_strdup ("--text-font-green");
              options[i++] = g_strdup_printf ("%d", textbox_text_color.green / 257);
              options[i++] = g_strdup ("--text-font-blue");
              options[i++] = g_strdup_printf ("%d", textbox_text_color.blue / 257);
              options[i++] = g_strdup ("--text-font-alpha");
              options[i++] = g_strdup_printf ("%d", textbox_text_alpha);
              if (textbox_border)
                {
                  options[i++] = g_strdup ("--textbox-border-show");
                  options[i++] = g_strdup (textbox_border);
                  options[i++] = g_strdup ("--textbox-border-red");
                  options[i++] = g_strdup_printf ("%d", textbox_border_color.red / 257);
                  options[i++] = g_strdup ("--textbox-border-green");
                  options[i++] = g_strdup_printf ("%d", textbox_border_color.green / 257);
                  options[i++] = g_strdup ("--textbox-border-blue");
                  options[i++] = g_strdup_printf ("%d", textbox_border_color.blue / 257);
                  options[i++] = g_strdup ("--textbox-border-alpha");
                  options[i++] = g_strdup_printf ("%d", textbox_border_alpha);
                }
            }
          if (verbose)
            {
              options[i++] = g_strdup ("--autoverbose");
              options[i++] = g_strdup (verbose);
            }
          options[i] = NULL;
			
          // Call splashy_config
          ret = splashy_config (options);
        }
      // Free all this memory
      i--;
      for (; i >= 0; i--)
        if (options[i])
          g_free (options[i]);
    }


  switch (ret)
    {
    case RETURN_OK:  // Don't tell anything, just redraw the list
      if (create == TRUE)
        draw_theme_list (TreeView);
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "");
      break;
    case RETURN_ERROR:  // Error
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("An error occured, the theme hasn't been created"));
      break;
    case RETURN_NEED_ROOT:  // Not enough priviledges
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, _("You need super-user priviledges to use this feature"));
      break;
    }
	
  gtk_widget_hide (CreateThemeDialog);
}

/**
 * Install a dragged-in theme.
 */
void
on_MainWindow_drag_data_received (GtkWidget * widget, GdkDragContext * dc, gint x, gint y, 
                                  GtkSelectionData * selection_data)
{
  GtkWidget *TreeView = glade_xml_get_widget (xml, "ThemeList");
  GtkWidget *Statusbar =  glade_xml_get_widget (xml, "Statusbar");
  gchar *text = g_filename_from_uri ((const gchar *) selection_data->data, NULL, NULL);
  gint len = strlen (text);
  gboolean freetext = FALSE;
  gchar *options[4];
	
  if (text[len - 1] == '\n')
    {
      text = g_strndup (text, len - 2);
      freetext = TRUE;
    }
  options[0] = "splashy_config";
  options[1] = "--install-theme";
  options[2] = text;
  options[3] = NULL;
  gint ret = splashy_config (options);

  switch (ret)
    {
    case RETURN_OK:  // Don't tell anything, just redraw the list
      draw_theme_list (TreeView);
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "");
      break;
    case RETURN_ERROR:  // Error
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "An error occured, the theme hasn't been installed");
      break;
    case RETURN_NEED_ROOT:  // Not enough priviledges
      gtk_statusbar_push (GTK_STATUSBAR (Statusbar), ret, "You need super-user priviledges to use this feature");
      break;
    }

  if (freetext)
    g_free (text);
}

/**
 * Display or not the field Progressbar Border.
 */
void
on_CheckbuttonProgressbarBorder_toggled (void)
{
  GtkWidget *ColorbuttonProgressbarBorder = glade_xml_get_widget (xml, "ColorbuttonProgressbarBorder");
  GtkWidget *SpinbuttonProgressbarBorderAlpha = glade_xml_get_widget (xml, "SpinbuttonProgressbarBorderAlpha");
  static gboolean state = FALSE;

  if (state)
    state = FALSE;
  else
    state = TRUE;

  gtk_widget_set_sensitive (ColorbuttonProgressbarBorder, state);
  gtk_widget_set_sensitive (SpinbuttonProgressbarBorderAlpha, state);
}

/**
 * Display or not the field Textbox.
 */
void
on_CheckbuttonTextbox_toggled (void)
{
  GtkWidget *FrameTextboxDimensions = glade_xml_get_widget (xml, "FrameTextboxDimensions");
  GtkWidget *FrameTextboxColor = glade_xml_get_widget (xml, "FrameTextboxColor");
  GtkWidget *FrameTextboxBorder = glade_xml_get_widget (xml, "FrameTextboxBorder");
  GtkWidget *FrameTextboxText = glade_xml_get_widget (xml, "FrameTextboxText");
  static gboolean state = FALSE;

  if (state)
    state = FALSE;
  else
    state = TRUE;

  gtk_widget_set_sensitive (FrameTextboxDimensions, state);
  gtk_widget_set_sensitive (FrameTextboxColor, state);
  gtk_widget_set_sensitive (FrameTextboxBorder, state);
  gtk_widget_set_sensitive (FrameTextboxText, state);
}

/**
 * Display or not the field Textbox Border.
 */
void
on_CheckbuttonTextboxBorder_toggled (void)
{
  GtkWidget *ColorbuttonTextboxBorder = glade_xml_get_widget (xml, "ColorbuttonTextboxBorder");
  GtkWidget *SpinbuttonTextboxBorderAlpha = glade_xml_get_widget (xml, "SpinbuttonTextboxBorderAlpha");
  static gboolean state = FALSE;

  if (state)
    state = FALSE;
  else
    state = TRUE;

  gtk_widget_set_sensitive (ColorbuttonTextboxBorder, state);
  gtk_widget_set_sensitive (SpinbuttonTextboxBorderAlpha, state);
}

/**
 * Check if the theme name is not NULL then allows the Apply button
 * to be pressed.
 */
void
on_EntryInfoName_changed (GtkWidget *widget)
{
  /*
   * TODO
   * Check the image file chooser in the same way and so
   * modify this function to not set the sensitivity of the
   * button to it but in on_CreateTheme_released.
   */
  GtkWidget *ApplyCreateTheme = glade_xml_get_widget (xml, "ApplyCreateTheme");
  gboolean NameFilled = FALSE;
  gchar *ret = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));

  if (strlen (ret) > 0)
    NameFilled = TRUE;
  else
    NameFilled = FALSE;

  gtk_widget_set_sensitive (ApplyCreateTheme, NameFilled);
  g_free (ret);
}

/**
 * When a character is typed in the Height entry,
 * check the Width entry then if they are non NULL, modify
 * the max value of the X, Y, Width and Height of the progressbar.
 */
void
on_EntryBackgroundHeight_changed (GtkWidget *Height)
{
  gboolean HeightNull = TRUE;
  gboolean WidthNull = TRUE;
  const gchar *percentage = _("(percentage)");
  const gchar *pixel = _("(pixel)");
  gdouble x_max;
  gdouble y_max;
  GtkWidget *Width = glade_xml_get_widget (xml, "EntryBackgroundWidth");
  GtkWidget *SpinX = glade_xml_get_widget (xml, "SpinbuttonProgressbarX");
  GtkWidget *SpinY = glade_xml_get_widget (xml, "SpinbuttonProgressbarY");
  GtkWidget *SpinWidth = glade_xml_get_widget (xml, "SpinbuttonProgressbarWidth");
  GtkWidget *SpinHeight = glade_xml_get_widget (xml, "SpinbuttonProgressbarHeight");
  GtkWidget *LabelPbUnit = glade_xml_get_widget (xml, "LabelProgressbarUnit");
  gchar *ret = g_strdup (gtk_entry_get_text (GTK_ENTRY (Height)));


  if (g_strtod (ret, NULL) > 0)
    HeightNull = FALSE;
  else
    HeightNull = TRUE;

  g_free (ret);
  ret = g_strdup (gtk_entry_get_text (GTK_ENTRY (Width)));

  if (g_strtod (ret, NULL) > 0)
    WidthNull = FALSE;
  else
    WidthNull = TRUE;

  if (HeightNull || WidthNull)
    {
      x_max = 100;
      y_max = 100;
      gtk_label_set_text (GTK_LABEL (LabelPbUnit), percentage);
    }
  else
    {
      x_max = g_strtod (gtk_entry_get_text (GTK_ENTRY (Width)), NULL);
      y_max = g_strtod (gtk_entry_get_text (GTK_ENTRY (Height)), NULL);
      gtk_label_set_text (GTK_LABEL (LabelPbUnit), pixel);
    }

  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinX), 0, x_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinY), 0, y_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinWidth), 1, x_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinHeight), 1, y_max);
}

/**
 * Counterpart of the on_EntryBackgroundHeight_changed 
 * callback for the Width.
 */
void
on_EntryBackgroundWidth_changed (GtkWidget *Width)
{
  gboolean HeightNull = TRUE;
  gboolean WidthNull = TRUE;
  const gchar *percentage = _("(percentage)");
  const gchar *pixel = _("(pixel)");
  gdouble x_max;
  gdouble y_max;
  GtkWidget *Height = glade_xml_get_widget (xml, "EntryBackgroundHeight");
  GtkWidget *SpinX = glade_xml_get_widget (xml, "SpinbuttonProgressbarX");
  GtkWidget *SpinY = glade_xml_get_widget (xml, "SpinbuttonProgressbarY");
  GtkWidget *SpinWidth = glade_xml_get_widget (xml, "SpinbuttonProgressbarWidth");
  GtkWidget *SpinHeight = glade_xml_get_widget (xml, "SpinbuttonProgressbarHeight");
  GtkWidget *LabelPbUnit = glade_xml_get_widget (xml, "LabelProgressbarUnit");
  gchar *ret = g_strdup (gtk_entry_get_text (GTK_ENTRY (Height)));

  if (g_strtod (ret, NULL) > 0)
    HeightNull = FALSE;
  else
    HeightNull = TRUE;

  g_free (ret);
  ret = g_strdup (gtk_entry_get_text (GTK_ENTRY (Width)));

  if (g_strtod (ret, NULL) > 0)
    WidthNull = FALSE;
  else
    WidthNull = TRUE;

  if (HeightNull || WidthNull)
    {
      x_max = 100;
      y_max = 100;
      gtk_label_set_text (GTK_LABEL (LabelPbUnit), percentage);
    }
  else
    {
      x_max = g_strtod (gtk_entry_get_text (GTK_ENTRY (Width)), NULL);
      y_max = g_strtod (gtk_entry_get_text (GTK_ENTRY (Height)), NULL);
      gtk_label_set_text (GTK_LABEL (LabelPbUnit), pixel);
    }

  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinX), 0, x_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinY), 0, y_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinWidth), 1, x_max);
  gtk_spin_button_set_range (GTK_SPIN_BUTTON (SpinHeight), 1, y_max);
}

void on_DialogClose_released (void)
{

  GtkWidget *Dialog = glade_xml_get_widget (xml, "Dialog");
  gtk_widget_hide (Dialog);
}
